from GFM314_Rx_tests.General.GFM314_base import RxTestsBase
from GFM314_Rx_tests.General.GFM314_utilities import RsException
from RsInstrument.Internal.InstrumentErrors import RsInstrException


class TC73(RxTestsBase):
    def __init__(self, resource_name: str):
        super().__init__(resource_name)
        super().reset()

        self.bs_class = None
        """Select base station class: Wide area ('WIDE'), medium area ('MED') or local area ('LOC')"""

        self.trigger_config = None
        """Select trigger configuration: Armed auto ('AAUT') or unchanged ('UNCH')"""
        self.marker_config = None
        """Select marker configuration: Radio frame start ('FRAM') or unchanged ('UNCH')"""

        self.frequency = None
        """Set frequency in MHz"""
        self.channel_bw = None
        """Set channel bandwidth in MHz"""
        self.scs = None
        """Set sub carrier spacing in kHz"""
        self.cell_id = None
        """Set the cell id"""
        self.ue_id = None
        """Set the user equipment id"""
        self.dmrs_pos = None
        """Set the DMRS type a position"""
        self.rb_offset = None
        """Set the shift of the allocated wanted signal in number of resource blocks"""

        self._check_device_suitability(['K62', 'K144'])
        """Minimum software option requirement"""

    def apply_configuration(self):
        """
        This function applies the configuration and sends the remote control commands to the connected device.
        :return:
        """
        try:
            self.io.write_str(':BB:NR5G:TCW:TC TS381411_TC73')
            if self.bs_class is not None:
                self.io.write_str(f':BB:NR5G:TCW:BSCL {self.bs_class}')

            super().instrument_settings(self.trigger_config, self.marker_config)

            if self.frequency is not None:
                self.io.write_float(':BB:NR5G:TCW:WS:RFFR ', self.frequency * 1000000)
            if self.channel_bw is not None:
                self.io.write_str(f':BB:NR5G:TCW:WS:CBW BW{self.channel_bw}')
            if self.scs is not None:
                self.io.write_str(f':BB:NR5G:TCW:WS:SCSP N{self.scs}')
            if self.cell_id is not None:
                self.io.write_int(':BB:NR5G:TCW:WS:CELL', self.cell_id)
            if self.ue_id is not None:
                self.io.write_int(':BB:NR5G:TCW:WS:UEID', self.ue_id)
            if self.dmrs_pos is not None:
                self.io.write_int(':BB:NR5G:TCW:WS:TAP', self.dmrs_pos)
            if self.rb_offset is not None:
                self.io.write_int(':BB:NR5G:TCW:WS:RBOF', self.rb_offset)

            super().tcw_apply()
        except RsInstrException as e:
            raise RsException(e.args[0])
