from GFM314_Rx_tests.General.GFM314_base import RxTestsBase
from GFM314_Rx_tests.General.GFM314_utilities import RsException
from RsInstrument.Internal.InstrumentErrors import RsInstrException
import csv


class TC76(RxTestsBase):
    def __init__(self, resource_name: str):
        super().__init__(resource_name)
        super().reset()
        self.frequency_5thharmonic = None
        """Upper frequency edge of the 5th harmonic [GHz]"""
        self.number_peaks = None
        """Set the number of peaks"""
        self.offset = None
        """Set offset in dB"""

    def apply_configuration(self):
        """
        This function applies the configuration and sends the remote control commands to the connected spectrum
        analyzer. :return:
        """
        try:
            self.io.write_str(':SYST:DISP:UPD ON')
            self.io.write_str(':SWE:MODE LIST')
            self.io.write_str(':INIT:CONT OFF')
            if self.number_peaks is not None:
                self.io.write_int(':CALC:PEAK:SUBR', self.number_peaks)
            if self.offset is not None:
                self.io.write_float(':DISP:WIND:SUBW:TRAC:Y:SCAL:RLEV:OFFS', self.offset)
            self.io.write_str(':SENS:LIST:RANG1:DEL')
            self.io.write_str(':SENS:LIST:RANG1:FREQ:STAR 30MHZ')
            self.io.write_str(':SENS:LIST:RANG1:FREQ:STOP 1GHZ')
            self.io.write_str(':SENS:LIST:RANG1:BAND 100KHZ')
            self.io.write_str(':SENS:LIST:RANG1:LIM:STAR -57')
            self.io.write_str(':SENS:LIST:RANG1:LIM:STOP -57')

            self.io.write_str(':SENS:LIST:RANG2:FREQ:STAR 1GHz')
            self.io.write_str(':SENS:LIST:RANG2:FREQ:STOP 12.75GHz')
            self.io.write_str(':SENS:LIST:RANG2:BAND 1MHZ')
            self.io.write_str(':SENS:LIST:RANG2:LIM:STAR -47')
            self.io.write_str(':SENS:LIST:RANG2:LIM:STOP -47')

            self.io.write_str(':SENS:LIST:RANG3:FREQ:STAR 12.75GHz')
            if self.frequency_5thharmonic is not None:
                self.io.write_float(':SENS:LIST:RANG3:FREQ:STOP ', self.frequency_5thharmonic * 1000000000)
            self.io.write_str(':SENS:LIST:RANG3:BAND 1MHZ')
            self.io.write_str(':SENS:LIST:RANG3:LIM:STAR -47')
            self.io.write_str(':SENS:LIST:RANG3:LIM:STOP -47')

            self.io.write_str(':SENS:LIST:XADJ;*WAI')
            self.io.write_str_with_opc(':INIT:SPUR')
        except RsInstrException as e:
            raise RsException(e.args[0])

    def return_results(self):
        """
        This function retuns the summary table of the spurious emissions measurement
        :return: Summary table of spurious emissions measurement
        """
        results = self.io.query_bin_or_ascii_float_list_with_opc('TRAC:DATA? LIST')
        header = ['Range Low [Hz]', 'Range Up [Hz]', 'RBW [Hz]', 'PeakFreq [Hz]', 'PowerAbs [dBm]', 'PowerRel [dB]',
                  'PowerDelta [dB]', 'LimitCheck [1/0]']
        return [header, results[1:9], results[276:284], results[551:559]]

    def csv_export(self):
        """
        Export spurious emission measurement results to csv file (project directory)
        :return:
        """
        with open('results_SPUR.csv', 'w', newline='') as csvfile:
            writer = csv.writer(csvfile, delimiter=';')
            results = self.return_results()
            entries = [results[0], results[1], results[2], results[3]]
            writer.writerows(entries)
